# ----------------------------------------------------------------------------
#  AndroidModule.cmake is meant to aid and abet the creation of android projects
#  using CMake.
#
#   When you include this at the top of your root CMakeLists.txt it will
#   define macros and set variables needed for generating Android ndk build
#   files.
#   Do something like the following:
#set(ANDROID_CMAKE_DIR ${CMAKE_SOURCE_DIR}/../android_cmake CACHE PATH 
#    "The android cmake scripts     directory")
#include(${ANDROID_CMAKE_DIR}/AndroidModule.cmake REQUIRED)
#
#   - initial version August 2010, for OpenCV port, 
#           by Ethan Rublee ethan.ruble@gmail.com
#   - Changed December 2010 for pcl_perception
#
# ----------------------------------------------------------------------------
#initial setup of Android specific variables.
set(arm_targets "armeabi-v7a" CACHE string "arm targets, armeabi-v7a is for floating point hardware support,armeabi is without")

set(ANDROID true)
set(JNI_DIR ${CMAKE_BINARY_DIR}/jni)
set(NDK_ROOT  "$ENV{HOME}/android-ndk-r4-crystax" CACHE STRING "the crystax ndk directory")
if(NOT EXISTS ${NDK_ROOT})
message(FATAL_ERROR "Cannot find your ndk root directory! please download and 
unzip the android ndk from crystax to the directory specified by NDK_ROOT
You may download the crystax ndk from: 
        http://www.crystax.net/android/ndk-r4.php" )
endif()

set(J  "2" CACHE STRING "how many processes for make -j <J>")

set(NDK_BUILD_ARGS  --directory=${CMAKE_BINARY_DIR} NDK_APPLICATION_MK=Application.mk -j${J} V=1  )


set(NDK_BUILD_CLEAN_ARGS  --directory=${CMAKE_BINARY_DIR} NDK_APPLICATION_MK=Application.mk  V=1 clean )



macro(gen_android_project name APP_MODULES APP_INCLUDES)
set(MY_LIB_LIBS "")
set(MY_LIB ${name})
set(MY_LIB_LIB_DIRS "-L${CMAKE_BINARY_DIR}/obj/local/armeabi-v7a -L${CMAKE_BINARY_DIR}/obj/local/armeabi" )
set(TAM "")
foreach(m ${APP_MODULES})
    set(TAM "${TAM} ${m}")
    set(MY_LIB_LIBS "${MY_LIB_LIBS} -l${m}")
endforeach()
set(APP_MODULES ${TAM})
set(MY_LIB_INCLUDE_DIRS "")
foreach(i ${APP_INCLUDES})
    set(MY_LIB_INCLUDE_DIRS "${MY_LIB_INCLUDE_DIRS} ${i}")
endforeach()
configure_file("${ANDROID_CMAKE_DIR}/mylib.config.mk.in" "${CMAKE_BINARY_DIR}/${MY_LIB}.config.mk")
configure_file("${ANDROID_CMAKE_DIR}/Android.mk.master.in" "${CMAKE_BINARY_DIR}/Android.mk")
configure_file("${ANDROID_CMAKE_DIR}/Application.mk.in" "${CMAKE_BINARY_DIR}/Application.mk")
configure_file("${ANDROID_CMAKE_DIR}/AndroidManifest.xml.in" "${CMAKE_BINARY_DIR}/AndroidManifest.xml")
configure_file("${ANDROID_CMAKE_DIR}/default.properties.in" "${CMAKE_BINARY_DIR}/default.properties")
configure_file("${ANDROID_CMAKE_DIR}/Android.mk.master.in" "${JNI_DIR}/Android.mk")
ADD_CUSTOM_COMMAND(
OUTPUT    ${name}
COMMAND   "${NDK_ROOT}/ndk-build" 
ARGS      ${NDK_BUILD_ARGS}
)

ADD_CUSTOM_TARGET(ndk ALL echo
  DEPENDS ${name}
)
endmacro()

macro(android_status)
message(STATUS "Make will use make -j${J} - for speeding up build - you may change this in the cache")
message(STATUS "The NDK directory is ${NDK_ROOT}")
message(STATUS "ndk-build args: ${NDK_BUILD_ARGS}")
message(STATUS "just run make - and grab some coffee or tea ;)")
message(STATUS "To use these libs, link to them from your android project. See the ${CMAKE_BINARY_DIR}/${MY_LIB}.config.mk")
endmacro()

#macros for making an android module
#name - name of the library
#lib_srcs - a list of the sources, either absolute or relative to
#           ${module_root_path}
#includes - the include directorys (absolute paths)
#
#Optional defines before calling macro:
#
# module_root_path - the absolute path of the srcs with a trailing / please
#                    for example set(module_root_path ${pcl_root}/)
#
# LOCAL_PATH - the absolute path of the module, if this is specified, then it
#              should be the same as module_root_path without the trailing /
#              set(LOCAL_PATH ${pcl_root})
#              this defaults to the ${CMAKE_CURRENT_BINARY_DIR}/${name} 
#              directory in this case the lib_srcs must be able to be resolved 
#              relative to this path.
#
#
macro(define_android_manual name lib_srcs includes type)
project(${name})
configure_file("${ANDROID_CMAKE_DIR}/Android.mk.master.in" "${CMAKE_CURRENT_BINARY_DIR}/Android.mk")
set(android_module_name ${name})
set(android_srcs "")
set(include_dirs "${includes}")

set(module_path ${CMAKE_CURRENT_BINARY_DIR}/${name})
configure_file("${ANDROID_CMAKE_DIR}/what.txt.in" "${module_path}/what.txt")
if(NOT DEFINED LOCAL_PATH)
set(LOCAL_PATH ${module_path})
endif()

foreach(f ${lib_srcs})
        file(RELATIVE_PATH n_f ${LOCAL_PATH} ${module_root_path}${f})
		set(android_srcs "${android_srcs} ${n_f}")
endforeach()
configure_file("${ANDROID_CMAKE_DIR}/Android.mk.${type}.in" "${module_path}/Android.mk")
endmacro()


macro(swig_wrapper_gen name)

    file(GLOB swig_srcs ${CMAKE_CURRENT_SOURCE_DIR}/src/*.i)

    file(GLOB SWIG_MAIN ${CMAKE_CURRENT_SOURCE_DIR}/src/${name}.i)
    
	ADD_CUSTOM_COMMAND(
	OUTPUT ${SWIG_C_DIR}
	DEPENDS   ${swig_srcs}
	COMMAND   mkdir
	ARGS  -p ${SWIG_C_DIR}
	)
	
	ADD_CUSTOM_COMMAND(
	OUTPUT ${SWIG_JAVA_DIR}
	DEPENDS   ${swig_srcs}
	COMMAND   mkdir
	ARGS  -p ${SWIG_JAVA_DIR}
	)
	
    ADD_CUSTOM_COMMAND(
        OUTPUT    ${name}_swig
        DEPENDS   ${swig_srcs} ${SWIG_JAVA_DIR} ${SWIG_C_DIR}
        COMMAND   swig
        ARGS -java -c++ -I${android_opencv_jni_root}/jni -package "${JAVA_ROOT_PACKAGE}.jni" -outdir ${SWIG_JAVA_DIR} -o ${SWIG_C_OUT} ${SWIG_MAIN}
    )

    ADD_CUSTOM_TARGET(${name}_swig_gen echo
      DEPENDS ${name}_swig
    )
    
    set(SWIG_OUT_FILES ${SWIG_OUT_FILES} ${SWIG_C_OUT})
endmacro()

macro(define_jni_module name)
    set(SWIG_JAVA_DIR ${JAVA_ROOT_DIR}/jni)
    set(SWIG_C_DIR ${CMAKE_CURRENT_SOURCE_DIR}/src/gen)
    set(SWIG_C_OUT ${SWIG_C_DIR}/${name}_swig.cpp)    
    swig_wrapper_gen(${name})
    file(GLOB lib_srcs "${CMAKE_CURRENT_SOURCE_DIR}/src/*.c*")
    set(lib_srcs "${lib_srcs} ${SWIG_C_OUT}")
	file(GLOB lib_int_hdrs "src/*.h*")
	define_android_manual(${name} "${lib_srcs}" "$(LOCAL_PATH)/include $(LOCAL_PATH)/src ${android_module_include_dirs} $(OPENCV_INCLUDES) $(ANDROID_OPENCV_INCLUDES)" jni)
endmacro()

macro(define_module name)
	file(GLOB lib_srcs "${CMAKE_CURRENT_SOURCE_DIR}/src/*.c*")
	file(GLOB lib_int_hdrs "${CMAKE_CURRENT_SOURCE_DIR}/src/*.h*")
	define_android_manual(${name} "${lib_srcs}" "$(LOCAL_PATH)/include $(LOCAL_PATH)/src ${android_module_include_dirs} $(OPENCV_INCLUDES)" static)
endmacro()

macro(rosbuild_add_library lib)
   # add_library(${lib} SHARED ${ARGN})
   define_android_manual( ${lib} "${ARGN}" 
    "${lib_includes}" static)

endmacro(rosbuild_add_library)



